@@ -1,7 +1,6 @@
 use strict; # -*- cperl -*-
 use warnings;
 
-use lib qw( ../../../../lib );
 
 =head1 NAME
 
@@ -27,10 +26,8 @@ use lib qw( ../../../../lib );
 
 package Algorithm::Evolutionary::Fitness::Knapsack;
 
-our ($VERSION) = ( '$Revision: 3.1 $ ' =~ / (\d+\.\d+)/ ) ;
-
 use Carp qw( croak );
-use base qw(Algorithm::Evolutionary::Fitness::String);
+use base qw(Algorithm::Evolutionary::Fitness::Base);
 
 =head2 new
 
@@ -51,21 +48,26 @@ sub new {
      croak "Profits and weights differ";
    }
 
-  #Instantiate superclass
-  my $self = $class->SUPER::new();
-  
-  $self->{'capacity'} = $capacity;
-  $self->{ 'rho' }  = $rho;
-  $self->{ 'profits'} = $profits_ref;
-  $self->{ 'weights'} = $weights_ref;
-
+  #Generate peaks
+  my $self = { capacity => $capacity,
+	       rho => $rho,
+	       profits => $profits_ref,
+	       weights => $weights_ref};
+  bless $self, $class;
   $self->initialize();
   return $self;
 }
 
-sub _really_apply {
+=head2 _apply
+
+Applies the instantiated problem to a chromosome
+
+=cut
+
+sub _apply {
     my $self = shift;
-    return  $self->knapsack( @_ );
+    my $individual = shift;
+    return  $self->knapsack( $individual->{_str});
 }
 
 =head2 knapsack
@@ -74,13 +76,14 @@ sub _really_apply {
 
 =cut
 
+our %cache;
+
 sub knapsack {
     my $self = shift;
     my $string = shift;
 
-    my $cache = $self->{'_cache'};
-    if ( $cache->{$string} ) {
-	return $cache->{$string};
+    if ( $cache{$string} ) {
+	return $cache{$string};
     }
     my $profit=0.0;
     my $weight=0.0;
@@ -92,8 +95,8 @@ sub knapsack {
       my $this_bit=substr ($string, $i, 1);
       
       if ($this_bit == 1)  {
-        $profit += $profits[$i];
-        $weight += $weights[$i];
+        $profit = $profit + $profits[$i];
+        $weight = $weight + $weights[$i];
       }
     }
     
@@ -103,20 +106,31 @@ sub knapsack {
     }
 
     #Y devolvemos la ganancia calculada
-    $cache->{$string} = $profit;
+    $cache{$string} = $profit;
     return $profit;
 }
 
+=head2 cached_evals
+
+Returns the number of keys in the evaluation cache
+
+=cut
+
+sub cached_evals {
+    return scalar keys %cache;
+}
+
+
 
 =head1 Copyright
   
   This file is released under the GPL. See the LICENSE file included in this distribution,
   or go to http://www.fsf.org/licenses/gpl.txt
 
-  CVS Info: $Date: 2010/09/24 08:39:07 $ 
-  $Header: /media/Backup/Repos/opeal/opeal/Algorithm-Evolutionary/lib/Algorithm/Evolutionary/Fitness/Knapsack.pm,v 3.1 2010/09/24 08:39:07 jmerelo Exp $ 
+  CVS Info: $Date: 2008/06/17 11:38:36 $ 
+  $Header: /cvsroot/opeal/Algorithm-Evolutionary/lib/Algorithm/Evolutionary/Fitness/Knapsack.pm,v 1.1 2008/06/17 11:38:36 jmerelo Exp $ 
   $Author: jmerelo $ 
-  $Revision: 3.1 $
+  $Revision: 1.1 $
   $Name $
 
 =cut
