@@ -1,9 +1,10 @@
-use strict; #-*-cperl-*-
+use strict;
 use warnings;
 
 =head1 NAME
 
-Algorithm::Evolutionary::Op::Base - Base class for Algorithm::Evolutionary operators,
+    Algorithm::Evolutionary::Op::Base - Base class for OPEAL operators; operators are any object with the "apply"
+             method, which does things to individuals or populations.
 
 =head1 SYNOPSIS
 
@@ -18,18 +19,15 @@ Algorithm::Evolutionary::Op::Base - Base class for Algorithm::Evolutionary opera
     my $ref = XMLin($xmlStr);
     my $op = Algorithm::Evolutionary::Op::Base->fromXML( $ref ); #Takes a hash of parsed XML and turns it into an operator    
 
-    print $op->asXML(); #prints it back in XML shape
+    print $op->asXML(); #print its back in XML shape
 
     print $op->rate();  #application rate; relative number of times it must be applied
-    print "Yes" if $op->check( 'Algorithm::Evolutionary::Individual::Bit_Vector' ); #Prints Yes, it can be applied to Bit_Vector individual
+    print "Yes" if $op->check( 'BinaryIndi' ); #Prints Yes, it can be applied to BinaryIndi
     print $op->arity(); #Prints 1, number of operands it can be applied to
 
 =head1 DESCRIPTION
 
-Base class for operators applied to Individuals and Populations and
-all the rest.  An operator is any object with the "apply" method,
-which does things to individuals or populations. It is intendedly
-quite general so that any genetic or population operator can fit in. 
+Base class for operators applied to Individuals and Populations and all the rest
 
 =head1 METHODS
 
@@ -39,20 +37,20 @@ package Algorithm::Evolutionary::Op::Base;
 
 use lib qw( ../.. ../../.. );
 
+use XML::Parser;
+use XML::Parser::EasyTree;
 use Memoize;
-memoize('arity'); #To speed up this frequent computation
+memoize('arity');
 
 use B::Deparse; #For serializing code
-use Algorithm::Evolutionary::Utils qw(parse_xml);
 
 use Carp;
-our ($VERSION) = ( '$Revision: 3.3 $ ' =~ / (\d+\.\d+)/ ) ;
-our %parameters;
+our $VERSION = ( '$Revision: 1.2 $ ' =~ /(\d+\.\d+)/ ) ;
+
 
 =head2 AUTOLOAD
 
-Automatically define accesors for instance variables. You should
-probably not worry about this unless you are going to subclass.
+Automatically define accesors for instance variables
 
 =cut
 
@@ -71,7 +69,7 @@ sub AUTOLOAD {
 
 }
 
-=head2 new( [$priority] [,$options_hash] )
+=head2 new
 
 Takes a hash with specific parameters for each subclass, creates the 
 object, and leaves subclass-specific assignments to subclasses
@@ -80,33 +78,16 @@ object, and leaves subclass-specific assignments to subclasses
 
 sub new {
   my $class = shift;
-  carp "Should be called from subclasses" if ( $class eq  __PACKAGE__ );
   my $rate = shift || 1;
   my $hash = shift; #No carp here, some operators do not need specific stuff
-  my $self = { rate => $rate,
-	       _arity => eval( "\$"."$class"."::ARITY" )}; # Create a reference
+  my $self = { rate => $rate }; # Create a reference
+  carp "Should be called from subclasses" if ( $class eq  __PACKAGE__ );
   bless $self, $class; # And bless it
   $self->set( $hash ) if $hash ;
   return $self;
 }
 
-=head2 create( [@operator_parameters] )
-
-Creates an operator via its default parameters. Probably obsolete
-
-=cut
-
-sub create {
-  my $class = shift;
-  my $self;
-  for my $p ( keys %parameters ) {
-    $self->{"_$p"} = shift || $parameters{$p}; # Default
-  }
-  bless $self, $class;
-  return $self;
-}
-
-=head2 fromXML()
+=head2 fromXML
 
 Takes a definition in the shape <op></op> and turns it into an object, 
 if it knows how to do it. The definition must have been processed using XML::Simple.
@@ -118,64 +99,57 @@ subclass via the "set" method.
 
 sub fromXML {
   my $class = shift;
-  my $xml = shift || croak "XML fragment missing ";
-  my $fragment; # Inner part of the XML
+  my $xml = shift || carp "XML fragment missing ";
   if ( ref $xml eq ''  ) { #We are receiving a string, parse it
-    $xml = parse_xml( $xml );
-    croak "Incorrect XML fragment" if !$xml->{'op'}; #
-    $fragment = $xml->{'op'};
-  } else {
-    $fragment = $xml;
+    my $p=new XML::Parser(Style=>'EasyTree');
+    $XML::Parser::EasyTree::Noempty=1;
+    $xml = $p->parse($xml);
   }
-  my $rate = shift;
-  if ( !defined $rate && $fragment->{'-rate'} ) {
-    $rate = $fragment->{'-rate'};
-  }
-  my $self = { rate => $rate }; # Create a reference
+  my $self = { rate => shift || $xml->[0]{attrib}{rate} }; # Create a reference
 
   if ( $class eq  __PACKAGE__ ) { #Deduct class from the XML
-    $class = $fragment->{'-name'} || shift || croak "Class name missing";
+    $class = $xml->[0]{attrib}{name} || shift || croak "Class name missing";
   }
   
   $class = "Algorithm::Evolutionary::Op::$class" if $class !~ /Algorithm::Evolutionary/;
   bless $self, $class; # And bless it
-  
-  my (%params, %code_fragments, %ops);
-  
-  for ( @{ (ref $fragment->{'param'} eq 'ARRAY')?
-	     $fragment->{'param'}:
-	       [ $fragment->{'param'}] } ) {
-    if  ( defined $_->{'-value'} ) {
-      $params{$_->{'-name'}} = $_->{'-value'};
-    } elsif ( $_->{'param'} ) {
-      my %params_hash;
-      for my $p ( @{ (ref $_->{'param'} eq 'ARRAY')?
-		       $_->{'param'}:
-			 [ $_->{'param'}] } ) {
-	$params_hash{ $p->{'-name'}} = $p->{'-value'};
-      }
-      $params{$_->{'-name'}} = \%params_hash;
-    }
+
+  my (%params, %codeFrags, %ops);
+  my $fragment;
+  if ( ( scalar @$xml > 1 ) || ! $xml->[0]{content}[0] ) { #Received from experiment or suchlike; already processed
+    $fragment = $xml;
+  }  else {
+    $fragment = $xml->[0]{content};
   }
-  
-  if ($fragment->{'code'} ) {
-    $code_fragments{$fragment->{'code'}->{'-type'}} = $fragment->{'code'}->{'src'};
-  }    
-       
-  for ( @{$fragment->{'op'}} ) { 
-    $ops{$_->{'-name'}} = [$_->{'-rate'}, $_];
+  for (@$fragment ) {
+    next if !defined  $_->{name};
+    if ( $_->{name} eq 'param' ) { 
+      if ( ! defined ( $_->{'content'}[0] ) ) {
+	$params{$_->{'attrib'}{'name'}} = $_->{'attrib'}{'value'};
+      } else {
+	$params{$_->{'attrib'}{'name'}} = $_->{'content'}
+      }
+    }
+
+    if ( $_->{name} eq 'code' ) { 
+      $codeFrags{$_->{'attrib'}{'type'}} = $_->{'content'}[0]{'content'}[0]{content};
+    }
+    
+    if ( $_->{name} eq 'op' ) { 
+      $ops{$_->{'attrib'}{'name'}} = [$_->{'attrib'}{'rate'}, $_->{'content'}, $_->{'attrib'}{'id'}];
+    }
   }
 
   #If the class is not loaded, we load it. The 
   eval "require $class" || croak "Can't find $class Module";
 
   #Let the class configure itself
-  $self->set( \%params, \%code_fragments, \%ops );
+  $self->set( \%params, \%codeFrags, \%ops );
   return $self;
 }
 
 
-=head2 asXML( [$id] )
+=head2 asXML
 
 Prints as XML, following the EvoSpec 0.2 XML specification. Should be
 called from derived classes, not by itself. Provides a default
@@ -187,8 +161,7 @@ name='foo' rate='1' E<gt> >.
 If there is not anything special, this takes also care of the instance
 variables different from C<rate>: they are inserted as C<param> within
 the XML file. In this case, C<param>s are void tags; if you want
-anything more fancy, you will have to override this method. An
-optional ID can be used.
+anything more fancy, you will have to override this method.
 
 =cut
 
@@ -206,21 +179,20 @@ sub asXML {
   } else {
     $str .= " >";
     for ( keys %$self ) {
-      next if !$self->{$_};
       if (!/\brate\b/ ) {
-	my ($paramName) = /_(\w+)/;
-	if ( ! ref $self->{$_}  ) {
-	  $str .= "\n\t<param name='$paramName' value='$self->{$_}' />";
-	} elsif ( ref $self->{$_} eq 'ARRAY' ) {
-	  for my $i ( @{$self->{$_}} ) {
-	    $str .= $i->asXML()."\n";
-	  }
-	} elsif ( ref $self->{$_} eq 'CODE' ) {
-	  my $deparse = B::Deparse->new;
-	  $str .="<code type='eval' language='perl'>\n<src><![CDATA[".$deparse->coderef2text($self->{$_})."]]>\n </src>\n</code>";
-	} elsif ( (ref $self->{$_} ) =~ 'Algorithm::Evolutionary' ) { #Composite object, I guess...
-	  $str .= $self->{$_}->asXML( $_ );
-	}
+		my ($paramName) = /_(\w+)/;
+		if ( ! ref $self->{$_}  ) {
+		  $str .= "\n\t<param name='$paramName' value='$self->{$_}' />";
+		} elsif ( ref $self->{$_} eq 'ARRAY' ) {
+		  for my $i ( @{$self->{$_}} ) {
+			$str .= $i->asXML()."\n";
+		  }
+		} elsif ( ref $self->{$_} eq 'CODE' ) {
+		  my $deparse = B::Deparse->new;
+		  $str .="<code type='eval' language='perl'>\n<src><![CDATA[".$deparse->coderef2text($self->{$_})."]]>\n </src>\n</code>";
+		} elsif ( (ref $self->{$_} ) =~ 'Algorithm::Evolutionary' ) { #Composite object, I guess...
+		$str .= $self->{$_}->asXML( $_ );
+		}
       }
     }
     $str .= "\n</op>";
@@ -228,7 +200,7 @@ sub asXML {
   return $str;
 }
 
-=head2 rate( [$rate] )
+=head2 rate
 
 Gets or sets the rate of application of the operator
 
@@ -240,9 +212,9 @@ sub rate {
   return $self;
 }
 
-=head2 check()
+=head2 check
 
-Check if the object the operator is applied to is in the correct
+Check if the object the operator is applied to has is in the correct
 class. 
 
 =cut
@@ -254,7 +226,7 @@ sub check {
   return $object->isa( $at ) ;
 }
 
-=head2 arity()
+=head2 arity
 
 Returns the arity, ie, the number of individuals it can be applied to
 
@@ -265,14 +237,14 @@ sub arity {
   return eval( "\$"."$class"."::ARITY" );
 }
 
-=head2 set( $options_hashref )
+=head2 set
 
 Converts the parameters passed as hash in instance variables. Default
 method, probably should be overriden by derived classes. If it is not,
 it sets the instance variables by prepending a C<_> to the keys of the
 hash. That is, 
-    $op->set( { foo => 3, bar => 6} );
-will set C<$op-E<gt>{_foo}> and  C<$op-E<gt>{_bar}> to the corresponding values
+  $op->set( { foo => 3, bar => } );
+will set C<$op->{_foo}> and  C<$op->{_bar}> to the corresponding values
 
 =cut
 
@@ -284,10 +256,7 @@ sub set {
   }
 }
 
-=head2 Known subclasses
-
-This is quite incomplete. Should be either generated automatically or
-suppressed altogether 
+=head1 Known subclasses
 
 =over 4
 
@@ -301,31 +270,23 @@ L<Algorithm::Evolutionary::Op::Mutation|Algorithm::Evolutionary::Op::Mutation>
 
 =item * 
 
-L<Algorithm::Evolutionary::Op::Mutation|Algorithm::Evolutionary::Op::IncMutation>
-
-=item * 
-
 L<Algorithm::Evolutionary::Op::BitFlip|Algorithm::Evolutionary::Op::BitFlip>
 
 =item * 
 
-L<Algorithm::Evolutionary::Op::GaussianMutation|Algorithm::Evolutionary::Op:GaussianMutation>
+L<Algorithm::Evolutionary::Op:GaussianMutation|Algorithm::Evolutionary::Op:GaussianMutation>
 
 =item * 
 
-L<Algorithm::Evolutionary::Op::Novelty_Mutation>
+L<Algorithm::Evolutionary::Op:Crossover|Algorithm::Evolutionary::Op:Crossover>
 
 =item * 
 
-L<Algorithm::Evolutionary::Op:Crossover>
+L<Algorithm::Evolutionary::Op:VectorCrossover|Algorithm::Evolutionary::Op:VectorCrossover>
 
 =item * 
 
-L<Algorithm::Evolutionary::Op::VectorCrossover|Algorithm::Evolutionary::Op:VectorCrossover>
-
-=item * 
-
-L<Algorithm::Evolutionary::Op::CX|Algorithm::Evolutionary::Op:CX>
+L<Algorithm::Evolutionary::Op:CX|Algorithm::Evolutionary::Op:CX>
 
 =item * 
 
@@ -349,21 +310,28 @@ L<Algorithm::Evolutionary::Op::DeltaTerm|Algorithm::Evolutionary::Op::DeltaTerm>
 L<Algorithm::Evolutionary::Op::Easy|Algorithm::Evolutionary::Op::Easy>
 
 =item * 
+L<Algorithm::Evolutionary::Op::FullAlgorithm|Algorithm::Evolutionary::Op::FullAlgorithm>
 
-L<Algorithm::Evolutionary::Op::FullAlgorithm>
 
 
 =back
 
 =head1 See Also
 
-The introduction to the XML format used here, L<XML>
+The introduction to the XML format used here,
+L<Algorithm::Evolutionary::XML.pod|Algorithm::Evolutionary::XML.pod>
 
 =head1 Copyright
   
   This file is released under the GPL. See the LICENSE file included in this distribution,
   or go to http://www.fsf.org/licenses/gpl.txt
 
+  CVS Info: $Date: 2008/04/01 08:24:15 $ 
+  $Header: /cvsroot/opeal/Algorithm-Evolutionary/lib/Algorithm/Evolutionary/Op/Base.pm,v 1.2 2008/04/01 08:24:15 jmerelo Exp $ 
+  $Author: jmerelo $ 
+  $Revision: 1.2 $
+  $Name $
+
 =cut
 
 "What???";
