@@ -3,11 +3,11 @@ use warnings;
 
 =head1 NAME
 
-Algorithm::Evolutionary::Op::CanonicalGA - Canonical Genetic Algorithm, with any representation
+    Algorithm::Evolutionary::Op::CanonicalGA - Canonical Genetic Algorithm, with any representation
+                 
 
 =head1 SYNOPSIS
 
-  # Straightforward instance, with all defaults (except for fitness function)
   my $algo = new Algorithm::Evolutionary::Op::CanonicalGA( $eval ); 
 
   #Define an easy single-generation algorithm with predefined mutation and crossover
@@ -36,29 +36,24 @@ population.
 
 package Algorithm::Evolutionary::Op::CanonicalGA;
 
-use lib qw(../../..);
-
-our $VERSION =   sprintf "%d.1%02d", q$Revision: 3.6 $ =~ /(\d+)\.(\d+)/g; # Hack for avoiding version mismatch
-
+our $VERSION = ( '$Revision: 1.1.1.1 $ ' =~ /(\d+\.\d+)/ ) ;
 
 use Carp;
+use Algorithm::Evolutionary::Wheel;
+use Algorithm::Evolutionary::Op::Bitflip;
+use Algorithm::Evolutionary::Op::QuadXOver;
 
-use Algorithm::Evolutionary qw(Wheel
-			       Op::Bitflip
-			       Op::QuadXOver );
-
-use base 'Algorithm::Evolutionary::Op::Easy';
+use Algorithm::Evolutionary::Op::Easy;
+our @ISA = qw(Algorithm::Evolutionary::Op::Easy);
 
 # Class-wide constants
 our $APPLIESTO =  'ARRAY';
 our $ARITY = 1;
 
-=head2 new( $fitness[, $selection_rate][,$operators_ref_to_array] )
+=head2 new
 
 Creates an algorithm, with the usual operators. Includes a default mutation
-and crossover, in case they are not passed as parameters. The first
-    element in the array ref should be an unary, and the second a
-    binary operator.
+and crossover, in case they are not passed as parameters
 
 =cut
 
@@ -81,12 +76,12 @@ sub new {
 
 }
 
-=head2 apply( $population) 
+=head2 apply
+
+Applies the algorithm to the population; checks that it receives a
+ref-to-array as input, croaks if it does not. Returns a sorted,
+culled, evaluated population for next generation.
 
-Applies a single generation of the algorithm to the population; checks
-that it receives a ref-to-array as input, croaks if it does
-not. Returns a sorted, culled, evaluated population for next
-generation.
 
 =cut
 
@@ -97,60 +92,46 @@ sub apply ($) {
 
   my $eval = $self->{_eval};
   for ( @$pop ) {
+    my $fitness;  #Evaluates only those that have no fitness
     if ( !defined ($_->Fitness() ) ) {
-	$_->evaluate( $eval );
+      $fitness = $eval->( $_ );
+      $_->Fitness( $fitness );
     }
   }
 
   my @newPop;
-  @$pop = sort { $b->{_fitness} <=> $a->{_fitness} } @$pop;
-  my @rates = map( $_->Fitness(), @$pop );
-
-  #Creates a roulette wheel from the op priorities. Theoretically,
-  #they might have changed 
-  my $popWheel= new Algorithm::Evolutionary::Wheel @rates;
+  my @rates;
+  for ( @$pop ) {
+    push @rates, $_->{_fitness};
+  }
+  my $popWheel=new Algorithm::Evolutionary::Wheel @rates;
   my $popSize = scalar @$pop;
   my @ops = @{$self->{_ops}};
   for ( my $i = 0; $i < $popSize*(1-$self->{_selrate})/2; $i ++ ) {
-      my $clone1 = $ops[0]->apply( $pop->[$popWheel->spin()] ); # This should be a mutation-like op
-      my $clone2 = $ops[0]->apply( $pop->[$popWheel->spin()] );
-      $ops[1]->apply( $clone1, $clone2 ); #This should be a
+    my $clone1 = $pop->[$popWheel->spin()]->clone();
+    my $clone2 = $pop->[$popWheel->spin()]->clone();
+    $clone1 = $ops[0]->apply($clone1 ); # This should be a mutation-like op
+    $clone2 = $ops[0]->apply( $clone2 );
+    $ops[1]->apply( $clone1, $clone2 ); #This should be a
                                           #crossover-like op
-      $clone1->evaluate( $eval );
-      $clone2->evaluate( $eval );
-      push @newPop, $clone1, $clone2;
+    $clone1->Fitness( $eval->( $clone1 ));
+    $clone2->Fitness( $eval->( $clone2 ));
+    push @newPop, $clone1, $clone2;
   }
   #Re-sort
   @{$pop}[$popSize*$self->{_selrate}..$popSize-1] =  @newPop;
   @$pop = sort { $b->{_fitness} <=> $a->{_fitness} } @$pop;
 }
 
-
-=head1 SEE ALSO
-
-=over 4
-
-=item L<Algorithm::Evolutionary::Op::Easy>
-
-=item L<Algorithm::Evolutionary::Wheel>
-
-=item L<Algorithm::Evolutionary::Fitness::Base>
-
-=back
-
-Probably you will also be able to find a
-    L<canonical-genetic-algorithm.pl> example within this
-    bundle. Check it out for usage examples
-
 =head1 Copyright
   
   This file is released under the GPL. See the LICENSE file included in this distribution,
   or go to http://www.fsf.org/licenses/gpl.txt
 
-  CVS Info: $Date: 2011/02/14 06:55:36 $ 
-  $Header: /media/Backup/Repos/opeal/opeal/Algorithm-Evolutionary/lib/Algorithm/Evolutionary/Op/CanonicalGA.pm,v 3.6 2011/02/14 06:55:36 jmerelo Exp $ 
+  CVS Info: $Date: 2008/02/12 17:49:39 $ 
+  $Header: /cvsroot/opeal/Algorithm-Evolutionary/lib/Algorithm/Evolutionary/Op/CanonicalGA.pm,v 1.1.1.1 2008/02/12 17:49:39 jmerelo Exp $ 
   $Author: jmerelo $ 
-  $Revision: 3.6 $
+  $Revision: 1.1.1.1 $
   $Name $
 
 =cut
