@@ -3,19 +3,23 @@ use warnings;
 
 =head1 NAME
 
-Algorithm::Evolutionary::Op::Mutation - Bitflip mutation, changes several bits in a bitstring, depending on the probability
+  Algorithm::Evolutionary::Op::Mutation - BitFlip mutation, changes several bits in a bitstring, depending on the probability
 
 =head1 SYNOPSIS
 
   use Algorithm::Evolutionary::Op::Mutation;
 
+  my $xmlStr=<<EOC;
+  <op name='Mutation' type='unary' rate='2'>
+    <param name='probability' value='0.5' />
+  </op>
+  EOC
+  my $ref = XMLin($xmlStr);
 
-  #Create from scratch
-  my $op = new Algorithm::Evolutionary::Op::Mutation (0.5 ); 
+  my $op = Algorithm::Evolutionary::Op::->fromXML( $ref );
+  print $op->asXML(), "\n*Arity ->", $op->arity(), "\n";
 
-  #All options
-  my $priority = 1;
-  my $mutation = new Algorithm::Evolutionary::Op::Mutation 1/$length, $priority;
+  my $op = new Algorithm::Evolutionary::Op::Mutation (0.5 ); #Create from scratch
 
 =head1 Base Class
 
@@ -29,11 +33,12 @@ Mutation operator for a GA
 
 package  Algorithm::Evolutionary::Op::Mutation;
 
-our ($VERSION) = ( '$Revision: 3.2 $ ' =~ /(\d+\.\d+)/ );
+our ($VERSION) = ( '$Revision: 1.2 $ ' =~ /(\d+\.\d+)/ );
 
 use Carp;
 
-use base 'Algorithm::Evolutionary::Op::Base';
+use Algorithm::Evolutionary::Op::Base;
+our @ISA = qw (Algorithm::Evolutionary::Op::Base);
 
 #Class-wide constants
 our $APPLIESTO =  'Algorithm::Evolutionary::Individual::BitString';
@@ -41,16 +46,10 @@ our $ARITY = 1;
 
 =head1 METHODS
 
-=head2 new( [$mutation_rate] [, $operator_probability] )
+=head2 new
 
-Creates a new mutation operator with a bitflip application rate, which
-defaults to 0.5, and an operator application rate (general for all
-ops), which defaults to 1. Application rate will be converted in
-runtime to application probability, which will eventually depend on
-the rates of all the other operators. For instance, if this operator's
-rate is one and there's another with rate=4, probability will be 20%
-for this one and 80% for the other; 1 in 5 new individuals will be
-generated using this and the rest using the other one.
+Creates a new mutation operator with a bitflip application rate, which defaults to 0.5,
+and an operator application rate (general for all ops), which defaults to 1.
 
 =cut
 
@@ -65,12 +64,11 @@ sub new {
 }
 
 
-=head2 create( [$operator_probability] )
+=head2 create
 
-Creates a new mutation operator with an application rate. Rate
-defaults to 0.5 (which is rather high, you should not rely on it).
+Creates a new mutation operator with an application rate. Rate defaults to 0.5 (which is rather bit, you should not rely on it).
 
-Called C<create> to distinguish from the classwide ctor, new. It just
+Called create to distinguish from the classwide ctor, new. It just
 makes simpler to create a Mutation Operator
 
 =cut
@@ -85,13 +83,12 @@ sub create {
   return $self;
 }
 
-=head2 apply( $chromosome )
+=head2 apply
 
 Applies mutation operator to a "Chromosome", a bitstring, really. Can be
 applied only to I<victims> with the C<_str> instance variable; 
 it checks before application that the operand is of type
-L<Algorithm::Evolutionary::Individual::BitString|Algorithm::Evolutionary::Individual::BitString>. 
-It returns the victim.
+L<Algorithm::Evolutionary::Individual::BitString|Algorithm::Evolutionary::Individual::BitString>. It returns the victim.
 
 =cut
 
@@ -101,12 +98,11 @@ sub apply ($;$) {
   my $victim = $arg->clone();
   croak "Incorrect type ".(ref $victim) if ! $self->check( $victim );
   for ( my $i = 0; $i < length( $victim->{_str} ); $i ++ ) {
-      if ( rand() < $self->{_mutRate} ) {
+	if ( rand() < $self->{_mutRate} ) {
 	  my $bit = $victim->Atom($i);
 	  $victim->Atom($i,  $bit?0:1 );
-      }
+	}
   }
-  $victim->{'_fitness'} = undef ;
   return $victim;
 }
 
@@ -115,5 +111,11 @@ sub apply ($;$) {
   This file is released under the GPL. See the LICENSE file included in this distribution,
   or go to http://www.fsf.org/licenses/gpl.txt
 
+  CVS Info: $Date: 2008/02/18 19:35:03 $ 
+  $Header: /cvsroot/opeal/Algorithm-Evolutionary/lib/Algorithm/Evolutionary/Op/Mutation.pm,v 1.2 2008/02/18 19:35:03 jmerelo Exp $ 
+  $Author: jmerelo $ 
+  $Revision: 1.2 $
+  $Name $
+
 =cut
 
